/* This testcase is part of GDB, the GNU debugger.

   Copyright 2012-2018 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see  <http://www.gnu.org/licenses/>.  */

class A {
 public:
  virtual ~A() { }
};

class B : public A {
 public:
  virtual ~B() { }

  int i;
  char c;
};

typedef int *int_ptr;

int
func (const A &a)
{
  int val = 10;
  int &int_ref = val;
  int_ptr ptr = &val;
  int_ptr &int_ptr_ref = ptr;
  B b;

  b.i = 10;
  b.c = 'a';

  return 0; /* Break here.  */
}

int
main ()
{
  A obj;
  return func (obj);
}
